/**
 * Copyright 2002-2013 Evgeny Gryaznov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.textmapper.tool.parser;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.List;
import java.util.ArrayList;
import org.textmapper.lapg.api.LexerRule;
import org.textmapper.tool.parser.TMLexer.ErrorReporter;
import org.textmapper.tool.parser.TMLexer.Lexems;
import org.textmapper.tool.parser.TMTree.TextSource;
import org.textmapper.tool.parser.ast.*;
import org.textmapper.tool.parser.TMLexer.LapgSymbol;

public class TMParser {

	public static class ParseException extends Exception {
		private static final long serialVersionUID = 1L;

		public ParseException() {
		}
	}

	private final ErrorReporter reporter;

	public TMParser(ErrorReporter reporter) {
		this.reporter = reporter;
	}


	private static final boolean DEBUG_SYNTAX = false;
	TextSource source;
	private static final int[] tmAction = TMLexer.unpack_int(286,
		"\uffff\uffff\uffff\uffff\uffff\uffff\ufffd\uffff\245\0\246\0\uffcd\uffff\264\0\26" +
		"\0\247\0\250\0\uffff\uffff\235\0\234\0\244\0\261\0\uff93\uffff\uff8b\uffff\uffff" +
		"\uffff\uff7f\uffff\15\0\uffff\uffff\251\0\uff4f\uffff\uffff\uffff\uffff\uffff\uffff" +
		"\uffff\uffff\uffff\6\0\uffff\uffff\uffff\uffff\uffff\uffff\uffff\uffff\16\0\uffff" +
		"\uffff\uffff\uffff\uff49\uffff\uffff\uffff\21\0\24\0\uffff\uffff\243\0\uff43\uffff" +
		"\uffff\uffff\uffff\uffff\11\0\262\0\20\0\uffff\uffff\uffff\uffff\uffff\uffff\uffff" +
		"\uffff\4\0\uff17\uffff\22\0\252\0\uffff\uffff\uff11\uffff\uffff\uffff\uff0b\uffff" +
		"\uffff\uffff\17\0\uffff\uffff\uff05\uffff\ufefd\uffff\ufef7\uffff\52\0\56\0\57\0" +
		"\55\0\23\0\uffff\uffff\2\0\256\0\257\0\255\0\uffff\uffff\uffff\uffff\240\0\uffff" +
		"\uffff\14\0\25\0\ufec5\uffff\uffff\uffff\104\0\uffff\uffff\uffff\uffff\61\0\uffff" +
		"\uffff\53\0\54\0\uffff\uffff\uffff\uffff\uffff\uffff\ufebd\uffff\111\0\114\0\115" +
		"\0\uffff\uffff\ufe8b\uffff\224\0\253\0\uffff\uffff\10\0\uffff\uffff\uffff\uffff\103" +
		"\0\51\0\60\0\uffff\uffff\41\0\42\0\35\0\36\0\ufe5f\uffff\33\0\34\0\40\0\43\0\45\0" +
		"\44\0\37\0\uffff\uffff\32\0\ufe1d\uffff\uffff\uffff\132\0\133\0\134\0\uffff\uffff" +
		"\ufde9\uffff\230\0\ufdb7\uffff\uffff\uffff\uffff\uffff\ufd7b\uffff\ufd4f\uffff\130" +
		"\0\131\0\uffff\uffff\112\0\113\0\uffff\uffff\225\0\uffff\uffff\106\0\107\0\105\0" +
		"\27\0\ufd23\uffff\uffff\uffff\30\0\31\0\ufce1\uffff\ufca7\uffff\uffff\uffff\137\0" +
		"\144\0\uffff\uffff\uffff\uffff\uffff\uffff\uffff\uffff\ufc9f\uffff\uffff\uffff\ufc63" +
		"\uffff\263\0\uffff\uffff\212\0\uffff\uffff\ufc15\uffff\151\0\ufc0d\uffff\153\0\ufbd3" +
		"\uffff\ufb97\uffff\172\0\175\0\177\0\ufb57\uffff\173\0\ufb15\uffff\ufad1\uffff\uffff" +
		"\uffff\ufa89\uffff\ufa5b\uffff\174\0\161\0\160\0\ufa2d\uffff\122\0\123\0\126\0\127" +
		"\0\ufa01\uffff\uf9c5\uffff\uffff\uffff\254\0\50\0\uffff\uffff\63\0\uf989\uffff\141" +
		"\0\143\0\136\0\uffff\uffff\135\0\145\0\uffff\uffff\uffff\uffff\166\0\uffff\uffff" +
		"\uffff\uffff\uffff\uffff\uf951\uffff\uffff\uffff\uf923\uffff\232\0\uffff\uffff\uffff" +
		"\uffff\uffff\uffff\uffff\uffff\121\0\uf8e7\uffff\uf8a9\uffff\156\0\uf86d\uffff\171" +
		"\0\157\0\uffff\uffff\203\0\uffff\uffff\216\0\217\0\176\0\uf82d\uffff\125\0\uffff" +
		"\uffff\uffff\uffff\uf7ff\uffff\75\0\65\0\uf7c3\uffff\140\0\226\0\165\0\164\0\uffff" +
		"\uffff\162\0\213\0\uffff\uffff\uffff\uffff\231\0\uffff\uffff\200\0\uf78d\uffff\201" +
		"\0\152\0\155\0\uf745\uffff\205\0\206\0\120\0\117\0\uffff\uffff\uffff\uffff\67\0\uf705" +
		"\uffff\163\0\uffff\uffff\233\0\116\0\77\0\100\0\101\0\102\0\uffff\uffff\71\0\73\0" +
		"\uffff\uffff\76\0\215\0\214\0\uffff\uffff\uffff\uffff\ufffe\uffff\ufffe\uffff");

	private static final short[] tmLalr = TMLexer.unpack_short(2348,
		"\45\uffff\10\1\36\1\37\1\46\1\47\1\50\1\51\1\52\1\53\1\54\1\55\1\56\1\57\1\60\1\61" +
		"\1\62\1\63\1\64\1\65\1\66\1\67\1\70\1\uffff\ufffe\2\uffff\3\uffff\20\uffff\36\uffff" +
		"\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff" +
		"\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff" +
		"\47\uffff\46\uffff\40\uffff\41\uffff\42\uffff\21\242\uffff\ufffe\22\uffff\66\uffff" +
		"\14\7\uffff\ufffe\15\uffff\14\260\22\260\24\260\66\260\uffff\ufffe\45\uffff\10\0" +
		"\36\0\37\0\46\0\47\0\50\0\51\0\52\0\53\0\54\0\55\0\56\0\57\0\60\0\61\0\62\0\63\0" +
		"\64\0\65\0\66\0\67\0\70\0\uffff\ufffe\16\uffff\21\241\uffff\ufffe\10\uffff\0\5\uffff" +
		"\ufffe\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61" +
		"\uffff\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50" +
		"\uffff\47\uffff\46\uffff\24\237\uffff\ufffe\10\uffff\0\3\uffff\ufffe\16\uffff\24" +
		"\236\uffff\ufffe\66\uffff\14\7\uffff\ufffe\12\uffff\17\25\22\25\uffff\ufffe\22\uffff" +
		"\17\62\uffff\ufffe\20\uffff\36\uffff\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff" +
		"\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff" +
		"\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\0\12\10\12\uffff\ufffe\13" +
		"\uffff\16\110\21\110\uffff\ufffe\6\uffff\35\uffff\36\uffff\37\uffff\70\uffff\67\uffff" +
		"\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff" +
		"\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\0\13\uffff" +
		"\ufffe\35\uffff\37\223\46\223\47\223\50\223\51\223\52\223\53\223\54\223\55\223\56" +
		"\223\57\223\60\223\61\223\62\223\63\223\64\223\65\223\66\223\67\223\70\223\uffff" +
		"\ufffe\15\uffff\16\uffff\20\uffff\21\uffff\22\uffff\26\uffff\27\uffff\30\uffff\33" +
		"\uffff\34\uffff\35\uffff\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff\64\uffff\63" +
		"\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff\53\uffff\52" +
		"\uffff\51\uffff\50\uffff\47\uffff\46\uffff\24\47\uffff\ufffe\1\uffff\0\74\10\74\20" +
		"\74\36\74\37\74\46\74\47\74\50\74\51\74\52\74\53\74\54\74\55\74\56\74\57\74\60\74" +
		"\61\74\62\74\63\74\64\74\65\74\66\74\67\74\70\74\uffff\ufffe\73\uffff\21\227\22\227" +
		"\35\227\37\227\46\227\47\227\50\227\51\227\52\227\53\227\54\227\55\227\56\227\57" +
		"\227\60\227\61\227\62\227\63\227\64\227\65\227\66\227\67\227\70\227\uffff\ufffe\6" +
		"\uffff\20\uffff\22\uffff\23\uffff\35\uffff\36\uffff\37\uffff\70\uffff\67\uffff\66" +
		"\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff\55" +
		"\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\72\uffff\11" +
		"\154\14\154\uffff\ufffe\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff\64\uffff\63" +
		"\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff\53\uffff\52" +
		"\uffff\51\uffff\50\uffff\47\uffff\46\uffff\7\124\uffff\ufffe\37\uffff\70\uffff\67" +
		"\uffff\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56" +
		"\uffff\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\7" +
		"\124\uffff\ufffe\15\uffff\16\uffff\20\uffff\21\uffff\22\uffff\26\uffff\27\uffff\30" +
		"\uffff\33\uffff\34\uffff\35\uffff\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff\64" +
		"\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff\53" +
		"\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\24\46\uffff\ufffe\13\uffff\0" +
		"\64\3\64\10\64\20\64\22\64\36\64\37\64\46\64\47\64\50\64\51\64\52\64\53\64\54\64" +
		"\55\64\56\64\57\64\60\64\61\64\62\64\63\64\64\64\65\64\66\64\67\64\70\64\72\64\uffff" +
		"\ufffe\56\uffff\14\142\16\142\uffff\ufffe\6\uffff\20\uffff\22\uffff\23\uffff\35\uffff" +
		"\36\uffff\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff" +
		"\61\uffff\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff" +
		"\50\uffff\47\uffff\46\uffff\72\uffff\11\154\24\154\uffff\ufffe\12\25\17\25\32\25" +
		"\6\26\11\26\14\26\22\26\23\26\24\26\30\26\31\26\33\26\34\26\35\26\36\26\37\26\43" +
		"\26\44\26\46\26\47\26\50\26\51\26\52\26\53\26\54\26\55\26\56\26\57\26\60\26\61\26" +
		"\62\26\63\26\64\26\65\26\66\26\67\26\70\26\72\26\uffff\ufffe\11\uffff\14\150\24\150" +
		"\uffff\ufffe\6\uffff\22\uffff\23\uffff\35\uffff\37\uffff\70\uffff\67\uffff\66\uffff" +
		"\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff\55\uffff" +
		"\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\72\uffff\11\154\14" +
		"\154\24\154\uffff\ufffe\6\uffff\22\uffff\23\uffff\35\uffff\36\uffff\37\uffff\70\uffff" +
		"\67\uffff\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff" +
		"\56\uffff\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff" +
		"\72\uffff\11\154\14\154\24\154\uffff\ufffe\34\uffff\6\167\11\167\14\167\22\167\23" +
		"\167\24\167\35\167\36\167\37\167\43\167\46\167\47\167\50\167\51\167\52\167\53\167" +
		"\54\167\55\167\56\167\57\167\60\167\61\167\62\167\63\167\64\167\65\167\66\167\67" +
		"\167\70\167\72\167\uffff\ufffe\33\uffff\6\202\11\202\14\202\22\202\23\202\24\202" +
		"\34\202\35\202\36\202\37\202\43\202\46\202\47\202\50\202\51\202\52\202\53\202\54" +
		"\202\55\202\56\202\57\202\60\202\61\202\62\202\63\202\64\202\65\202\66\202\67\202" +
		"\70\202\72\202\uffff\ufffe\44\uffff\6\204\11\204\14\204\22\204\23\204\24\204\33\204" +
		"\34\204\35\204\36\204\37\204\43\204\46\204\47\204\50\204\51\204\52\204\53\204\54" +
		"\204\55\204\56\204\57\204\60\204\61\204\62\204\63\204\64\204\65\204\66\204\67\204" +
		"\70\204\72\204\uffff\ufffe\30\uffff\31\uffff\6\210\11\210\14\210\22\210\23\210\24" +
		"\210\33\210\34\210\35\210\36\210\37\210\43\210\44\210\46\210\47\210\50\210\51\210" +
		"\52\210\53\210\54\210\55\210\56\210\57\210\60\210\61\210\62\210\63\210\64\210\65" +
		"\210\66\210\67\210\70\210\72\210\uffff\ufffe\35\uffff\22\220\37\220\46\220\47\220" +
		"\50\220\51\220\52\220\53\220\54\220\55\220\56\220\57\220\60\220\61\220\62\220\63" +
		"\220\64\220\65\220\66\220\67\220\70\220\uffff\ufffe\35\uffff\22\222\37\222\46\222" +
		"\47\222\50\222\51\222\52\222\53\222\54\222\55\222\56\222\57\222\60\222\61\222\62" +
		"\222\63\222\64\222\65\222\66\222\67\222\70\222\uffff\ufffe\37\uffff\70\uffff\67\uffff" +
		"\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff" +
		"\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\7\124\uffff" +
		"\ufffe\6\uffff\20\uffff\22\uffff\23\uffff\35\uffff\36\uffff\37\uffff\70\uffff\67" +
		"\uffff\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56" +
		"\uffff\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\72" +
		"\uffff\11\154\14\154\uffff\ufffe\6\uffff\20\uffff\22\uffff\23\uffff\35\uffff\36\uffff" +
		"\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff" +
		"\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff" +
		"\47\uffff\46\uffff\72\uffff\11\154\14\154\uffff\ufffe\3\uffff\0\66\10\66\20\66\22" +
		"\66\36\66\37\66\46\66\47\66\50\66\51\66\52\66\53\66\54\66\55\66\56\66\57\66\60\66" +
		"\61\66\62\66\63\66\64\66\65\66\66\66\67\66\70\66\72\66\uffff\ufffe\35\uffff\37\220" +
		"\46\220\47\220\50\220\51\220\52\220\53\220\54\220\55\220\56\220\57\220\60\220\61" +
		"\220\62\220\63\220\64\220\65\220\66\220\67\220\70\220\21\223\uffff\ufffe\6\uffff" +
		"\22\uffff\23\uffff\35\uffff\36\uffff\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff" +
		"\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff" +
		"\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\43\uffff\72\uffff\11\154\24" +
		"\154\uffff\ufffe\6\uffff\20\uffff\22\uffff\23\uffff\35\uffff\36\uffff\37\uffff\70" +
		"\uffff\67\uffff\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57" +
		"\uffff\56\uffff\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46" +
		"\uffff\72\uffff\11\154\14\154\24\154\uffff\ufffe\6\uffff\22\uffff\23\uffff\35\uffff" +
		"\36\uffff\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff\64\uffff\63\uffff\62\uffff" +
		"\61\uffff\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff\53\uffff\52\uffff\51\uffff" +
		"\50\uffff\47\uffff\46\uffff\72\uffff\11\154\14\154\24\154\uffff\ufffe\34\uffff\6" +
		"\170\11\170\14\170\22\170\23\170\24\170\35\170\36\170\37\170\43\170\46\170\47\170" +
		"\50\170\51\170\52\170\53\170\54\170\55\170\56\170\57\170\60\170\61\170\62\170\63" +
		"\170\64\170\65\170\66\170\67\170\70\170\72\170\uffff\ufffe\35\uffff\22\221\37\221" +
		"\46\221\47\221\50\221\51\221\52\221\53\221\54\221\55\221\56\221\57\221\60\221\61" +
		"\221\62\221\63\221\64\221\65\221\66\221\67\221\70\221\uffff\ufffe\6\uffff\20\uffff" +
		"\22\uffff\23\uffff\35\uffff\36\uffff\37\uffff\70\uffff\67\uffff\66\uffff\65\uffff" +
		"\64\uffff\63\uffff\62\uffff\61\uffff\60\uffff\57\uffff\56\uffff\55\uffff\54\uffff" +
		"\53\uffff\52\uffff\51\uffff\50\uffff\47\uffff\46\uffff\72\uffff\11\154\14\154\uffff" +
		"\ufffe\22\uffff\0\70\10\70\20\70\36\70\37\70\46\70\47\70\50\70\51\70\52\70\53\70" +
		"\54\70\55\70\56\70\57\70\60\70\61\70\62\70\63\70\64\70\65\70\66\70\67\70\70\70\72" +
		"\70\uffff\ufffe\30\uffff\31\uffff\6\211\11\211\14\211\22\211\23\211\24\211\33\211" +
		"\34\211\35\211\36\211\37\211\43\211\44\211\46\211\47\211\50\211\51\211\52\211\53" +
		"\211\54\211\55\211\56\211\57\211\60\211\61\211\62\211\63\211\64\211\65\211\66\211" +
		"\67\211\70\211\72\211\uffff\ufffe\34\207\6\207\11\207\14\207\22\207\23\207\24\207" +
		"\35\207\36\207\37\207\43\207\46\207\47\207\50\207\51\207\52\207\53\207\54\207\55" +
		"\207\56\207\57\207\60\207\61\207\62\207\63\207\64\207\65\207\66\207\67\207\70\207" +
		"\72\207\uffff\ufffe\72\uffff\0\72\10\72\20\72\36\72\37\72\46\72\47\72\50\72\51\72" +
		"\52\72\53\72\54\72\55\72\56\72\57\72\60\72\61\72\62\72\63\72\64\72\65\72\66\72\67" +
		"\72\70\72\uffff\ufffe");

	private static final short[] lapg_sym_goto = TMLexer.unpack_short(139,
		"\0\2\4\17\31\31\31\45\51\55\57\64\70\102\107\117\124\147\160\212\226\240\241\245" +
		"\251\260\263\264\271\300\327\354\u0129\u0131\u0139\u0140\u0141\u0142\u0144\u0183" +
		"\u01c1\u01ff\u023e\u027c\u02ba\u02f8\u0336\u0374\u03b2\u03f3\u0432\u0471\u04af\u04ed" +
		"\u052b\u056a\u05a8\u05e6\u05e6\u05f2\u05f3\u05f4\u05f5\u05f7\u05f9\u05fb\u05fc\u05fe" +
		"\u05ff\u0601\u061b\u063c\u063f\u0641\u0645\u0647\u0648\u064a\u064c\u064e\u064f\u0650" +
		"\u0651\u0653\u0654\u0656\u0658\u0659\u065b\u065d\u065f\u0660\u0662\u0663\u0665\u0667" +
		"\u0667\u066c\u0671\u0677\u067d\u0687\u068e\u0699\u06a4\u06b0\u06be\u06cc\u06d7\u06e5" +
		"\u06f4\u0700\u0703\u0712\u0725\u0731\u0732\u0739\u0741\u0742\u0743\u0745\u0748\u074b" +
		"\u0757\u076c\u076d\u076f\u0770\u0771\u0772\u0773\u0774\u0775\u0778\u0779\u0783\u0784" +
		"\u0785");

	private static final short[] lapg_sym_from = TMLexer.unpack_short(1925,
		"\u011a\u011b\125\174\1\6\22\40\50\62\114\126\220\237\346\1\6\50\54\62\114\220\237" +
		"\310\346\107\136\204\242\253\255\301\302\327\336\337\356\135\213\216\303\25\44\45" +
		"\65\251\331\43\70\77\146\246\70\122\146\231\35\37\60\117\233\236\250\354\355\u0107" +
		"\21\126\162\172\225\27\71\123\126\162\172\225\233\70\130\146\246\374\1\6\50\61\62" +
		"\101\114\126\162\172\204\220\225\237\242\301\302\336\356\30\123\126\162\172\225\322" +
		"\324\366\20\31\33\100\126\135\162\172\204\216\225\242\253\255\266\301\302\327\332" +
		"\333\334\336\337\344\356\361\204\241\242\253\255\301\302\327\336\337\344\356\53\72" +
		"\74\155\172\226\326\331\u010c\u0113\317\126\162\172\225\126\162\172\225\126\162\172" +
		"\225\265\376\u0116\265\376\u0116\246\126\162\172\225\262\126\162\172\225\256\341" +
		"\u0102\107\126\136\143\162\172\204\225\241\242\253\255\267\270\301\302\325\327\336" +
		"\337\344\352\356\1\6\25\45\50\62\101\114\134\136\204\220\237\242\255\301\302\327" +
		"\336\337\356\1\2\6\13\22\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134\136\142" +
		"\150\151\162\172\175\201\204\206\207\210\220\225\236\237\241\242\243\253\255\266" +
		"\274\301\302\306\314\320\323\327\332\333\334\336\337\344\346\356\371\372\u010c\1" +
		"\6\50\62\114\220\237\346\1\6\50\62\114\220\237\346\1\6\50\62\114\220\237\327\264" +
		"\3\23\1\2\6\13\22\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134\135\136\142" +
		"\150\151\162\172\175\201\204\206\207\210\216\220\225\236\237\241\242\243\253\255" +
		"\266\274\301\302\306\314\320\323\327\332\333\334\336\337\344\346\356\371\372\u010c" +
		"\1\2\6\13\22\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134\136\142\150\151\162" +
		"\172\175\201\204\206\207\210\220\225\236\237\240\241\242\243\253\255\266\274\301" +
		"\302\306\314\320\323\327\332\333\334\336\337\344\346\356\371\372\u010c\1\2\6\13\22" +
		"\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134\136\142\150\151\162\172\175\201" +
		"\204\206\207\210\220\225\236\237\240\241\242\243\253\255\266\274\301\302\306\314" +
		"\320\323\327\332\333\334\336\337\344\346\356\371\372\u010c\1\2\6\13\22\25\32\36\45" +
		"\50\52\61\62\76\101\107\114\115\126\134\135\136\142\150\151\162\172\175\201\204\206" +
		"\207\210\216\220\225\236\237\241\242\243\253\255\266\274\301\302\306\314\320\323" +
		"\327\332\333\334\336\337\344\346\356\371\372\u010c\1\2\6\13\22\25\32\36\45\50\52" +
		"\61\62\76\101\107\114\115\126\133\134\136\142\150\151\162\172\175\201\204\206\207" +
		"\210\220\225\236\237\241\242\243\253\255\266\274\301\302\306\314\320\323\327\332" +
		"\333\334\336\337\344\346\356\371\372\u010c\1\2\6\13\22\25\32\36\45\50\52\61\62\76" +
		"\101\107\114\115\126\133\134\136\142\150\151\162\172\175\201\204\206\207\210\220" +
		"\225\236\237\241\242\243\253\255\266\274\301\302\306\314\320\323\327\332\333\334" +
		"\336\337\344\346\356\371\372\u010c\1\2\6\13\22\25\32\36\45\50\52\61\62\76\101\107" +
		"\114\115\126\133\134\136\142\150\151\162\172\175\201\204\206\207\210\220\225\236" +
		"\237\241\242\243\253\255\266\274\301\302\306\314\320\323\327\332\333\334\336\337" +
		"\344\346\356\371\372\u010c\1\2\6\13\22\25\32\36\45\50\52\61\62\76\101\107\114\115" +
		"\126\133\134\136\142\150\151\162\172\175\201\204\206\207\210\220\225\236\237\241" +
		"\242\243\253\255\266\274\301\302\306\314\320\323\327\332\333\334\336\337\344\346" +
		"\356\371\372\u010c\1\2\6\13\22\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134" +
		"\136\142\150\151\162\172\175\201\204\206\207\210\220\225\232\236\237\241\242\243" +
		"\253\255\266\274\301\302\306\314\320\323\327\332\333\334\336\337\344\346\356\371" +
		"\372\u010c\1\2\6\13\22\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134\136\142" +
		"\150\151\162\172\175\201\204\206\207\210\220\225\236\237\241\242\243\253\255\266" +
		"\274\301\302\306\314\320\323\327\332\333\334\336\337\344\346\356\371\372\u0108\u010c" +
		"\1\2\6\13\22\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134\135\136\142\150\151" +
		"\162\172\175\201\204\205\206\207\210\216\220\225\236\237\241\242\243\253\255\266" +
		"\274\301\302\306\314\320\323\327\332\333\334\336\337\344\346\356\371\372\u0108\u010c" +
		"\1\2\6\13\22\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134\135\136\142\150\151" +
		"\162\172\175\201\204\206\207\210\216\220\225\236\237\241\242\243\253\255\266\274" +
		"\301\302\306\314\320\323\327\332\333\334\336\337\344\346\356\371\372\u010c\1\2\6" +
		"\13\22\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134\135\136\142\150\151\162" +
		"\172\175\201\204\206\207\210\216\220\225\236\237\241\242\243\253\255\266\274\301" +
		"\302\306\314\320\323\327\332\333\334\336\337\344\346\356\371\372\u010c\1\2\6\13\22" +
		"\25\32\36\45\50\52\61\62\76\101\107\114\115\126\134\136\142\150\151\162\172\175\201" +
		"\204\206\207\210\220\225\236\237\241\242\243\253\255\266\274\301\302\306\314\320" +
		"\323\327\332\333\334\336\337\344\346\356\371\372\u0108\u010c\1\2\6\13\22\25\32\36" +
		"\45\50\52\61\62\76\101\107\114\115\126\134\136\142\150\151\162\172\175\201\204\206" +
		"\207\210\220\225\236\237\241\242\243\253\255\266\274\301\302\306\314\320\323\327" +
		"\332\333\334\336\337\344\346\356\371\372\u0108\u010c\0\1\2\6\13\22\25\32\36\45\50" +
		"\52\61\62\76\101\107\114\115\126\134\136\142\150\151\162\172\175\201\204\206\207" +
		"\210\220\225\236\237\241\242\243\253\255\266\274\301\302\306\314\320\323\327\332" +
		"\333\334\336\337\344\346\356\371\372\u010c\1\2\6\13\20\22\25\32\36\45\50\52\61\62" +
		"\73\76\101\107\114\115\126\134\136\142\150\151\162\172\175\201\204\206\207\210\220" +
		"\225\236\237\241\242\243\253\255\266\274\301\302\306\314\320\323\327\332\333\334" +
		"\336\337\344\346\356\371\372\u010c\1\2\6\13\22\25\32\36\42\45\50\52\61\62\76\101" +
		"\107\114\115\126\134\136\142\150\151\162\172\175\201\204\206\207\210\220\225\236" +
		"\237\241\242\243\253\255\266\274\301\302\306\314\320\323\327\332\333\334\336\337" +
		"\344\346\356\371\372\u010c\1\2\6\13\22\25\32\36\45\50\52\61\62\63\76\101\107\114" +
		"\115\126\134\136\142\150\151\162\172\175\201\204\206\207\210\220\225\236\237\241" +
		"\242\243\253\255\266\274\301\302\306\314\320\323\327\332\333\334\336\337\344\346" +
		"\356\371\372\u010c\204\242\253\255\301\302\327\336\337\344\356\u010a\202\0\0\25\45" +
		"\44\65\20\73\3\3\23\25\25\45\61\76\101\107\136\142\151\204\207\210\241\242\253\255" +
		"\266\274\301\302\323\327\332\334\336\337\344\356\1\6\50\62\114\175\201\204\206\220" +
		"\236\237\242\243\253\255\266\301\302\314\320\327\332\333\334\336\337\344\346\356" +
		"\371\372\u010c\100\135\216\126\162\126\162\172\225\125\174\61\61\101\61\101\61\101" +
		"\231\361\u0108\61\101\76\150\306\76\151\107\107\136\107\136\135\216\133\107\136\175" +
		"\175\314\201\371\204\242\301\302\356\204\242\301\302\356\204\242\301\302\336\356" +
		"\204\242\301\302\336\356\204\242\253\255\301\302\327\336\337\356\204\242\253\301" +
		"\302\336\356\204\242\253\255\301\302\327\336\337\344\356\204\242\253\255\301\302" +
		"\327\336\337\344\356\204\242\253\255\266\301\302\327\336\337\344\356\204\242\253" +
		"\255\266\301\302\327\332\334\336\337\344\356\204\242\253\255\266\301\302\327\332" +
		"\334\336\337\344\356\204\242\253\255\301\302\327\336\337\344\356\204\242\253\255" +
		"\266\301\302\327\332\334\336\337\344\356\204\242\253\255\266\301\302\327\332\333" +
		"\334\336\337\344\356\204\241\242\253\255\301\302\327\336\337\344\356\107\136\241" +
		"\107\136\204\241\242\253\255\270\301\302\327\336\337\344\356\107\136\143\204\241" +
		"\242\253\255\267\270\301\302\325\327\336\337\344\352\356\204\241\242\253\255\301" +
		"\302\327\336\337\344\356\243\1\6\50\62\114\220\237\1\6\50\62\114\220\237\346\6\52" +
		"\70\146\2\13\32\2\13\32\204\242\253\255\301\302\327\336\337\344\356\u010a\1\6\25" +
		"\45\50\62\101\114\134\136\204\220\237\242\255\301\302\327\336\337\356\3\20\73\162" +
		"\100\231\310\361\u010a\207\210\274\232\204\242\253\255\301\302\327\336\337\356\52" +
		"\6");

	private static final short[] lapg_sym_to = TMLexer.unpack_short(1925,
		"\u011c\u011d\153\153\4\4\37\60\4\4\4\155\4\4\4\5\5\5\74\5\5\5\5\360\5\133\133\240" +
		"\240\240\240\240\240\240\240\240\240\204\301\302\356\42\63\42\63\336\372\62\111\125" +
		"\111\332\112\150\112\306\55\57\75\147\313\315\335\u0105\u0106\u010e\36\156\156\156" +
		"\156\50\115\151\157\157\157\157\314\113\174\113\333\333\6\6\6\76\6\76\6\160\160\160" +
		"\241\6\160\6\241\241\241\241\241\51\152\161\161\161\161\365\367\u010b\32\52\54\126" +
		"\162\126\162\162\242\126\162\242\242\242\242\242\242\242\242\242\242\242\242\242" +
		"\242\u0108\243\243\243\243\243\243\243\243\243\243\243\243\73\116\120\224\227\305" +
		"\370\373\u0116\u0117\363\163\163\163\163\164\164\164\164\165\165\165\165\347\347" +
		"\u0118\350\350\u0119\334\166\166\166\166\345\167\167\167\167\344\344\344\134\170" +
		"\134\134\170\170\134\170\134\134\134\134\134\134\134\134\134\134\134\134\134\134" +
		"\134\7\7\7\7\7\7\7\7\7\7\7\7\7\7\7\7\7\7\7\7\7\10\17\10\17\40\43\17\56\43\10\70\77" +
		"\10\121\77\121\10\146\171\202\121\121\221\121\171\171\10\10\244\10\121\121\10\171" +
		"\10\10\121\244\10\244\244\244\121\244\244\221\10\10\121\244\244\10\244\244\244\244" +
		"\10\244\10\10\10\11\11\11\11\11\11\11\11\12\12\12\12\12\12\12\12\13\13\13\13\13\13" +
		"\13\371\346\22\22\10\17\10\17\40\43\17\56\43\10\70\77\10\121\77\121\10\146\171\202" +
		"\205\121\121\221\121\171\171\10\10\244\10\121\121\205\10\171\10\10\121\244\10\244" +
		"\244\244\121\244\244\221\10\10\121\244\244\10\244\244\244\244\10\244\10\10\10\10" +
		"\17\10\17\40\43\17\56\43\10\70\77\10\121\77\121\10\146\171\202\121\121\221\121\171" +
		"\171\10\10\244\10\121\121\10\171\10\10\320\121\244\10\244\244\244\121\244\244\221" +
		"\10\10\121\244\244\10\244\244\244\244\10\244\10\10\10\10\17\10\17\40\43\17\56\43" +
		"\10\70\77\10\121\77\121\10\146\171\202\121\121\221\121\171\171\10\10\244\10\121\121" +
		"\10\171\10\10\321\121\244\10\244\244\244\121\244\244\221\10\10\121\244\244\10\244" +
		"\244\244\244\10\244\10\10\10\10\17\10\17\40\43\17\56\43\10\70\77\10\121\77\121\10" +
		"\146\171\202\206\121\121\221\121\171\171\10\10\244\10\121\121\206\10\171\10\10\121" +
		"\244\10\244\244\244\121\244\244\221\10\10\121\244\244\10\244\244\244\244\10\244\10" +
		"\10\10\10\17\10\17\40\43\17\56\43\10\70\77\10\121\77\121\10\146\171\175\202\121\121" +
		"\221\121\171\171\10\10\244\10\121\121\10\171\10\10\121\244\10\244\244\244\121\244" +
		"\244\221\10\10\121\244\244\10\244\244\244\244\10\244\10\10\10\10\17\10\17\40\43\17" +
		"\56\43\10\70\77\10\121\77\121\10\146\171\176\202\121\121\221\121\171\171\10\10\244" +
		"\10\121\121\10\171\10\10\121\244\10\244\244\244\121\244\244\221\10\10\121\244\244" +
		"\10\244\244\244\244\10\244\10\10\10\10\17\10\17\40\43\17\56\43\10\70\77\10\121\77" +
		"\121\10\146\171\177\202\121\121\221\121\171\171\10\10\244\10\121\121\10\171\10\10" +
		"\121\244\10\244\244\244\121\244\244\221\10\10\121\244\244\10\244\244\244\244\10\244" +
		"\10\10\10\10\17\10\17\40\43\17\56\43\10\70\77\10\121\77\121\10\146\171\200\202\121" +
		"\121\221\121\171\171\10\10\244\10\121\121\10\171\10\10\121\244\10\244\244\244\121" +
		"\244\244\221\10\10\121\244\244\10\244\244\244\244\10\244\10\10\10\10\17\10\17\40" +
		"\43\17\56\43\10\70\77\10\121\77\121\10\146\171\202\121\121\221\121\171\171\10\10" +
		"\244\10\121\121\10\171\311\10\10\121\244\10\244\244\244\121\244\244\221\10\10\121" +
		"\244\244\10\244\244\244\244\10\244\10\10\10\10\17\10\17\40\43\17\56\43\10\70\77\10" +
		"\121\77\121\10\146\171\202\121\121\221\121\171\171\10\10\244\10\121\121\10\171\10" +
		"\10\121\244\10\244\244\244\121\244\244\221\10\10\121\244\244\10\244\244\244\244\10" +
		"\244\10\10\u010f\10\10\17\10\17\40\43\17\56\43\10\70\77\10\121\77\121\10\146\171" +
		"\202\207\121\121\221\121\171\171\10\10\244\274\10\121\121\207\10\171\10\10\121\244" +
		"\10\244\244\244\121\244\244\221\10\10\121\244\244\10\244\244\244\244\10\244\10\10" +
		"\u0110\10\10\17\10\17\40\43\17\56\43\10\70\77\10\121\77\121\10\146\171\202\210\121" +
		"\121\221\121\171\171\10\10\244\10\121\121\210\10\171\10\10\121\244\10\244\244\244" +
		"\121\244\244\221\10\10\121\244\244\10\244\244\244\244\10\244\10\10\10\10\17\10\17" +
		"\40\43\17\56\43\10\70\77\10\121\77\121\10\146\171\202\211\121\121\221\121\171\171" +
		"\10\10\244\10\121\121\211\10\171\10\10\121\244\10\244\244\244\121\244\244\221\10" +
		"\10\121\244\244\10\244\244\244\244\10\244\10\10\10\10\17\10\17\40\43\17\56\43\10" +
		"\70\77\10\121\77\121\10\146\171\202\121\121\221\121\171\171\10\10\244\10\121\121" +
		"\10\171\10\10\121\244\10\244\244\244\121\244\244\221\10\10\121\244\244\10\244\244" +
		"\244\244\10\244\10\10\u0111\10\10\17\10\17\40\43\17\56\43\10\70\77\10\121\77\121" +
		"\10\146\171\202\121\121\221\121\171\171\10\10\244\10\121\121\10\171\10\10\121\244" +
		"\10\244\244\244\121\244\244\221\10\10\121\244\244\10\244\244\244\244\10\244\10\10" +
		"\u0112\10\2\10\17\10\17\40\43\17\56\43\10\70\77\10\121\77\121\10\146\171\202\121" +
		"\121\221\121\171\171\10\10\244\10\121\121\10\171\10\10\121\244\10\244\244\244\121" +
		"\244\244\221\10\10\121\244\244\10\244\244\244\244\10\244\10\10\10\10\17\10\17\33" +
		"\40\43\17\56\43\10\70\77\10\33\121\77\121\10\146\171\202\121\121\221\121\171\171" +
		"\10\10\244\10\121\121\10\171\10\10\121\244\10\244\244\244\121\244\244\221\10\10\121" +
		"\244\244\10\244\244\244\244\10\244\10\10\10\10\17\10\17\40\43\17\56\61\43\10\70\77" +
		"\10\121\77\121\10\146\171\202\121\121\221\121\171\171\10\10\244\10\121\121\10\171" +
		"\10\10\121\244\10\244\244\244\121\244\244\221\10\10\121\244\244\10\244\244\244\244" +
		"\10\244\10\10\10\10\17\10\17\40\43\17\56\43\10\70\77\10\107\121\77\121\10\146\171" +
		"\202\121\121\221\121\171\171\10\10\244\10\121\121\10\171\10\10\121\244\10\244\244" +
		"\244\121\244\244\221\10\10\121\244\244\10\244\244\244\244\10\244\10\10\10\245\245" +
		"\245\245\245\245\245\245\245\245\245\245\237\u011a\3\44\65\64\110\34\34\23\24\41" +
		"\45\46\66\100\122\100\135\135\216\122\246\276\276\322\246\246\246\246\276\246\246" +
		"\366\246\374\374\246\246\246\246\14\14\14\14\14\232\235\247\275\14\316\14\247\330" +
		"\247\247\247\247\247\232\364\247\247\247\247\247\247\247\u0103\247\235\u010d\316" +
		"\127\212\212\172\225\173\173\230\230\154\231\101\102\131\103\103\104\104\307\u0109" +
		"\u0113\105\105\123\222\357\124\223\136\137\214\140\140\213\303\201\141\141\233\234" +
		"\362\236\u010c\250\326\354\355\u0107\251\251\251\251\251\252\252\252\252\u0100\252" +
		"\253\253\253\253\253\253\254\254\254\254\254\254\254\254\254\254\255\327\337\255" +
		"\255\255\255\256\256\256\341\256\256\341\256\341\u0102\256\257\257\257\257\257\257" +
		"\257\257\257\257\257\260\260\260\260\351\260\260\260\260\260\260\260\261\261\261" +
		"\261\261\261\261\261\375\377\261\261\261\261\262\262\262\262\262\262\262\262\262" +
		"\262\262\262\262\262\263\263\263\263\263\263\263\263\263\263\263\264\264\264\264" +
		"\264\264\264\264\264\264\264\264\264\264\265\265\265\265\265\265\265\265\265\376" +
		"\265\265\265\265\265\266\323\266\266\266\266\266\266\266\266\266\266\142\142\324" +
		"\143\143\267\325\267\267\267\352\267\267\267\267\267\267\267\144\144\217\144\144" +
		"\144\144\144\217\144\144\144\217\144\144\144\144\217\144\270\270\270\270\270\270" +
		"\270\270\270\270\270\270\331\u011b\26\67\106\145\304\317\15\15\15\15\15\15\15\u0104" +
		"\27\71\114\220\20\31\53\21\21\21\271\271\271\271\271\271\271\271\271\271\271\u0114" +
		"\16\16\47\47\16\16\132\16\203\215\272\16\16\272\342\272\272\342\272\342\272\25\35" +
		"\117\226\130\310\361\u010a\u0115\277\300\353\312\273\273\340\343\273\273\343\273" +
		"\u0101\273\72\30");

	private static final short[] lapg_rlen = TMLexer.unpack_short(181,
		"\1\0\5\4\4\3\1\0\7\4\3\3\4\1\2\4\3\1\2\3\1\1\1\3\3\2\1\1\1\1\1\1\1\1\1\1\1\1\1\0" +
		"\3\1\1\2\2\1\1\1\3\1\0\1\0\1\0\1\0\1\0\10\3\2\3\1\1\1\1\3\1\3\1\3\1\1\2\2\1\1\6\5" +
		"\5\4\2\1\0\3\2\2\1\1\1\1\1\4\4\1\3\1\0\2\1\2\1\3\1\1\3\1\0\3\2\2\1\1\3\4\3\3\2\1" +
		"\2\2\1\1\1\1\2\1\3\3\1\2\1\3\3\3\1\3\1\3\6\6\2\2\1\2\1\1\1\2\5\2\2\3\1\3\1\1\1\0" +
		"\5\1\0\3\1\1\1\1\1\1\3\3\5\1\1\1\1\1\3\1\1");

	private static final short[] lapg_rlex = TMLexer.unpack_short(181,
		"\175\175\74\74\74\74\176\176\75\75\76\77\100\101\101\102\102\103\103\104\104\105" +
		"\106\107\107\110\110\111\111\111\111\111\111\111\111\111\111\111\177\177\111\112" +
		"\113\113\113\114\114\114\115\200\200\201\201\202\202\203\203\204\204\116\116\117" +
		"\120\121\121\121\121\122\123\123\124\125\125\126\126\126\127\127\130\130\130\130" +
		"\131\205\205\131\131\131\131\131\132\132\132\133\133\134\134\206\206\135\136\136" +
		"\137\137\140\141\141\207\207\142\142\142\142\142\143\143\143\144\144\145\145\145" +
		"\146\146\146\147\147\150\150\150\151\151\152\152\152\153\154\154\155\155\155\155" +
		"\155\155\156\156\156\157\160\160\161\161\161\162\163\163\164\164\210\210\164\211" +
		"\211\164\164\165\165\165\165\166\166\167\167\170\170\170\171\172\172\173\174");

	protected static final String[] lapg_syms = new String[] {
		"eoi",
		"regexp",
		"scon",
		"icon",
		"_skip",
		"_skip_comment",
		"'%'",
		"'::='",
		"'::'",
		"'|'",
		"'='",
		"'=>'",
		"';'",
		"'.'",
		"','",
		"':'",
		"'['",
		"']'",
		"'('",
		"'(?!'",
		"')'",
		"'}'",
		"'<'",
		"'>'",
		"'*'",
		"'+'",
		"'+='",
		"'?'",
		"'&'",
		"'@'",
		"error",
		"ID",
		"Ltrue",
		"Lfalse",
		"Lnew",
		"Lseparator",
		"Las",
		"Limport",
		"Linline",
		"Lprio",
		"Lshift",
		"Lreturns",
		"Linput",
		"Lleft",
		"Lright",
		"Lnonassoc",
		"Lnoeoi",
		"Lsoft",
		"Lclass",
		"Linterface",
		"Lvoid",
		"Lspace",
		"Llayout",
		"Llanguage",
		"Llalr",
		"Llexer",
		"Lparser",
		"Lreduce",
		"code",
		"'{'",
		"input",
		"header",
		"lexer_section",
		"parser_section",
		"parsing_algorithm",
		"imports",
		"import_",
		"options",
		"option",
		"identifier",
		"symref",
		"type",
		"type_part_list",
		"type_part",
		"pattern",
		"lexer_parts",
		"lexer_part",
		"named_pattern",
		"lexeme",
		"lexem_transition",
		"lexem_attrs",
		"lexem_attribute",
		"state_selector",
		"state_list",
		"stateref",
		"lexer_state",
		"grammar_parts",
		"grammar_part",
		"nonterm",
		"nonterm_type",
		"priority_kw",
		"directive",
		"inputs",
		"inputref",
		"references",
		"references_cs",
		"rules",
		"rule_list",
		"rule0",
		"rhsPrefix",
		"rhsSuffix",
		"rhsParts",
		"rhsPart",
		"rhsAnnotated",
		"rhsAssignment",
		"rhsOptional",
		"rhsCast",
		"rhsUnordered",
		"rhsClass",
		"rhsPrimary",
		"rhsAnnotations",
		"annotations",
		"annotation_list",
		"annotation",
		"negative_la",
		"negative_la_clause",
		"expression",
		"literal",
		"expression_list",
		"map_entries",
		"map_separator",
		"name",
		"qualified_id",
		"command",
		"syntax_problem",
		"importsopt",
		"parsing_algorithmopt",
		"type_part_listopt",
		"typeopt",
		"lexem_transitionopt",
		"iconopt",
		"lexem_attrsopt",
		"commandopt",
		"identifieropt",
		"Lnoeoiopt",
		"rhsSuffixopt",
		"map_entriesopt",
		"expression_listopt",
	};

	public interface Tokens extends Lexems {
		// non-terminals
		public static final int input = 60;
		public static final int header = 61;
		public static final int lexer_section = 62;
		public static final int parser_section = 63;
		public static final int parsing_algorithm = 64;
		public static final int imports = 65;
		public static final int import_ = 66;
		public static final int options = 67;
		public static final int option = 68;
		public static final int identifier = 69;
		public static final int symref = 70;
		public static final int type = 71;
		public static final int type_part_list = 72;
		public static final int type_part = 73;
		public static final int pattern = 74;
		public static final int lexer_parts = 75;
		public static final int lexer_part = 76;
		public static final int named_pattern = 77;
		public static final int lexeme = 78;
		public static final int lexem_transition = 79;
		public static final int lexem_attrs = 80;
		public static final int lexem_attribute = 81;
		public static final int state_selector = 82;
		public static final int state_list = 83;
		public static final int stateref = 84;
		public static final int lexer_state = 85;
		public static final int grammar_parts = 86;
		public static final int grammar_part = 87;
		public static final int nonterm = 88;
		public static final int nonterm_type = 89;
		public static final int priority_kw = 90;
		public static final int directive = 91;
		public static final int inputs = 92;
		public static final int inputref = 93;
		public static final int references = 94;
		public static final int references_cs = 95;
		public static final int rules = 96;
		public static final int rule_list = 97;
		public static final int rule0 = 98;
		public static final int rhsPrefix = 99;
		public static final int rhsSuffix = 100;
		public static final int rhsParts = 101;
		public static final int rhsPart = 102;
		public static final int rhsAnnotated = 103;
		public static final int rhsAssignment = 104;
		public static final int rhsOptional = 105;
		public static final int rhsCast = 106;
		public static final int rhsUnordered = 107;
		public static final int rhsClass = 108;
		public static final int rhsPrimary = 109;
		public static final int rhsAnnotations = 110;
		public static final int annotations = 111;
		public static final int annotation_list = 112;
		public static final int annotation = 113;
		public static final int negative_la = 114;
		public static final int negative_la_clause = 115;
		public static final int expression = 116;
		public static final int literal = 117;
		public static final int expression_list = 118;
		public static final int map_entries = 119;
		public static final int map_separator = 120;
		public static final int name = 121;
		public static final int qualified_id = 122;
		public static final int command = 123;
		public static final int syntax_problem = 124;
		public static final int importsopt = 125;
		public static final int parsing_algorithmopt = 126;
		public static final int type_part_listopt = 127;
		public static final int typeopt = 128;
		public static final int lexem_transitionopt = 129;
		public static final int iconopt = 130;
		public static final int lexem_attrsopt = 131;
		public static final int commandopt = 132;
		public static final int identifieropt = 133;
		public static final int Lnoeoiopt = 134;
		public static final int rhsSuffixopt = 135;
		public static final int map_entriesopt = 136;
		public static final int expression_listopt = 137;
	}

	/**
	 * -3-n   Lookahead (state id)
	 * -2     Error
	 * -1     Shift
	 * 0..n   Reduce (rule index)
	 */
	protected static int tmAction(int state, int symbol) {
		int p;
		if (tmAction[state] < -2) {
			for (p = -tmAction[state] - 3; tmLalr[p] >= 0; p += 2) {
				if (tmLalr[p] == symbol) {
					break;
				}
			}
			return tmLalr[p + 1];
		}
		return tmAction[state];
	}

	protected static int tmGoto(int state, int symbol) {
		int min = lapg_sym_goto[symbol], max = lapg_sym_goto[symbol + 1] - 1;
		int i, e;

		while (min <= max) {
			e = (min + max) >> 1;
			i = lapg_sym_from[e];
			if (i == state) {
				return lapg_sym_to[e];
			} else if (i < state) {
				min = e + 1;
			} else {
				max = e - 1;
			}
		}
		return -1;
	}

	protected int tmHead;
	protected LapgSymbol[] tmStack;
	protected LapgSymbol tmNext;
	protected TMLexer tmLexer;

	private Object parse(TMLexer lexer, int initialState, int finalState) throws IOException, ParseException {

		tmLexer = lexer;
		tmStack = new LapgSymbol[1024];
		tmHead = 0;
		int lapg_symbols_ok = 4;

		tmStack[0] = new LapgSymbol();
		tmStack[0].state = initialState;
		tmNext = tmLexer.next();

		while (tmStack[tmHead].state != finalState) {
			int action = tmAction(tmStack[tmHead].state, tmNext.symbol);

			if (action >= 0) {
				reduce(action);
			} else if (action == -1) {
				shift();
				lapg_symbols_ok++;
			}

			if (action == -2 || tmStack[tmHead].state == -1) {
				if (restore()) {
					if (lapg_symbols_ok >= 4) {
						reporter.error(tmNext.offset, tmNext.endoffset, tmNext.line,
								MessageFormat.format("syntax error before line {0}", tmLexer.getTokenLine()));
					}
					if (lapg_symbols_ok <= 1) {
						tmNext = tmLexer.next();
					}
					lapg_symbols_ok = 0;
					continue;
				}
				if (tmHead < 0) {
					tmHead = 0;
					tmStack[0] = new LapgSymbol();
					tmStack[0].state = initialState;
				}
				break;
			}
		}

		if (tmStack[tmHead].state != finalState) {
			if (lapg_symbols_ok >= 4) {
				reporter.error(tmNext.offset, tmNext.endoffset, tmNext.line,
						MessageFormat.format("syntax error before line {0}",
								tmLexer.getTokenLine()));
			}
			throw new ParseException();
		}
		return tmStack[tmHead - 1].value;
	}

	protected boolean restore() {
		if (tmNext.symbol == 0) {
			return false;
		}
		while (tmHead >= 0 && tmGoto(tmStack[tmHead].state, 30) == -1) {
			dispose(tmStack[tmHead]);
			tmStack[tmHead] = null;
			tmHead--;
		}
		if (tmHead >= 0) {
			tmStack[++tmHead] = new LapgSymbol();
			tmStack[tmHead].symbol = 30;
			tmStack[tmHead].value = null;
			tmStack[tmHead].state = tmGoto(tmStack[tmHead - 1].state, 30);
			tmStack[tmHead].line = tmNext.line;
			tmStack[tmHead].offset = tmNext.offset;
			tmStack[tmHead].endoffset = tmNext.endoffset;
			return true;
		}
		return false;
	}

	protected void shift() throws IOException {
		tmStack[++tmHead] = tmNext;
		tmStack[tmHead].state = tmGoto(tmStack[tmHead - 1].state, tmNext.symbol);
		if (DEBUG_SYNTAX) {
			System.out.println(MessageFormat.format("shift: {0} ({1})", lapg_syms[tmNext.symbol], tmLexer.current()));
		}
		if (tmStack[tmHead].state != -1 && tmNext.symbol != 0) {
			tmNext = tmLexer.next();
		}
	}

	protected void reduce(int rule) {
		LapgSymbol lapg_gg = new LapgSymbol();
		lapg_gg.value = (lapg_rlen[rule] != 0) ? tmStack[tmHead + 1 - lapg_rlen[rule]].value : null;
		lapg_gg.symbol = lapg_rlex[rule];
		lapg_gg.state = 0;
		if (DEBUG_SYNTAX) {
			System.out.println("reduce to " + lapg_syms[lapg_rlex[rule]]);
		}
		LapgSymbol startsym = (lapg_rlen[rule] != 0) ? tmStack[tmHead + 1 - lapg_rlen[rule]] : tmNext;
		lapg_gg.line = startsym.line;
		lapg_gg.offset = startsym.offset;
		lapg_gg.endoffset = (lapg_rlen[rule] != 0) ? tmStack[tmHead].endoffset : tmNext.offset;
		applyRule(lapg_gg, rule, lapg_rlen[rule]);
		for (int e = lapg_rlen[rule]; e > 0; e--) {
			cleanup(tmStack[tmHead]);
			tmStack[tmHead--] = null;
		}
		tmStack[++tmHead] = lapg_gg;
		tmStack[tmHead].state = tmGoto(tmStack[tmHead - 1].state, lapg_gg.symbol);
	}

	@SuppressWarnings("unchecked")
	protected void applyRule(LapgSymbol lapg_gg, int rule, int ruleLength) {
		switch (rule) {
			case 2:  // input ::= header importsopt options lexer_section parser_section
				 lapg_gg.value = new TmaInput(((TmaHeader)tmStack[tmHead - 4].value), ((List<TmaImport>)tmStack[tmHead - 3].value), ((List<TmaOptionPart>)tmStack[tmHead - 2].value), ((List<ITmaLexerPart>)tmStack[tmHead - 1].value), ((List<ITmaGrammarPart>)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 3:  // input ::= header importsopt options lexer_section
				 lapg_gg.value = new TmaInput(((TmaHeader)tmStack[tmHead - 3].value), ((List<TmaImport>)tmStack[tmHead - 2].value), ((List<TmaOptionPart>)tmStack[tmHead - 1].value), ((List<ITmaLexerPart>)tmStack[tmHead].value), ((List<ITmaGrammarPart>)null), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 4:  // input ::= header importsopt lexer_section parser_section
				 lapg_gg.value = new TmaInput(((TmaHeader)tmStack[tmHead - 3].value), ((List<TmaImport>)tmStack[tmHead - 2].value), ((List<TmaOptionPart>)null), ((List<ITmaLexerPart>)tmStack[tmHead - 1].value), ((List<ITmaGrammarPart>)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 5:  // input ::= header importsopt lexer_section
				 lapg_gg.value = new TmaInput(((TmaHeader)tmStack[tmHead - 2].value), ((List<TmaImport>)tmStack[tmHead - 1].value), ((List<TmaOptionPart>)null), ((List<ITmaLexerPart>)tmStack[tmHead].value), ((List<ITmaGrammarPart>)null), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 8:  // header ::= Llanguage name '(' name ')' parsing_algorithmopt ';'
				 lapg_gg.value = new TmaHeader(((TmaName)tmStack[tmHead - 5].value), ((TmaName)tmStack[tmHead - 3].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 9:  // header ::= Llanguage name parsing_algorithmopt ';'
				 lapg_gg.value = new TmaHeader(((TmaName)tmStack[tmHead - 2].value), ((TmaName)null), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 10:  // lexer_section ::= '::' Llexer lexer_parts
				 lapg_gg.value = ((List<ITmaLexerPart>)tmStack[tmHead].value); 
				break;
			case 11:  // parser_section ::= '::' Lparser grammar_parts
				 lapg_gg.value = ((List<ITmaGrammarPart>)tmStack[tmHead].value); 
				break;
			case 13:  // imports ::= import_
				 lapg_gg.value = new ArrayList<TmaImport>(16); ((List<TmaImport>)lapg_gg.value).add(((TmaImport)tmStack[tmHead].value)); 
				break;
			case 14:  // imports ::= imports import_
				 ((List<TmaImport>)tmStack[tmHead - 1].value).add(((TmaImport)tmStack[tmHead].value)); 
				break;
			case 15:  // import_ ::= Limport ID scon ';'
				 lapg_gg.value = new TmaImport(((String)tmStack[tmHead - 2].value), ((String)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 16:  // import_ ::= Limport scon ';'
				 lapg_gg.value = new TmaImport(((String)null), ((String)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 17:  // options ::= option
				 lapg_gg.value = new ArrayList<TmaOptionPart>(16); ((List<TmaOptionPart>)lapg_gg.value).add(((TmaOptionPart)tmStack[tmHead].value)); 
				break;
			case 18:  // options ::= options option
				 ((List<TmaOptionPart>)tmStack[tmHead - 1].value).add(((TmaOptionPart)tmStack[tmHead].value)); 
				break;
			case 19:  // option ::= ID '=' expression
				 lapg_gg.value = new TmaOption(((String)tmStack[tmHead - 2].value), ((ITmaExpression)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 21:  // identifier ::= ID
				 lapg_gg.value = new TmaIdentifier(((String)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 22:  // symref ::= ID
				 lapg_gg.value = new TmaSymref(((String)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 23:  // type ::= '(' scon ')'
				 lapg_gg.value = ((String)tmStack[tmHead - 1].value); 
				break;
			case 24:  // type ::= '(' type_part_list ')'
				 lapg_gg.value = source.getText(tmStack[tmHead - 2].offset+1, tmStack[tmHead].endoffset-1); 
				break;
			case 41:  // pattern ::= regexp
				 lapg_gg.value = new TmaPattern(((String)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 42:  // lexer_parts ::= lexer_part
				 lapg_gg.value = new ArrayList<ITmaLexerPart>(64); ((List<ITmaLexerPart>)lapg_gg.value).add(((ITmaLexerPart)tmStack[tmHead].value)); 
				break;
			case 43:  // lexer_parts ::= lexer_parts lexer_part
				 ((List<ITmaLexerPart>)tmStack[tmHead - 1].value).add(((ITmaLexerPart)tmStack[tmHead].value)); 
				break;
			case 44:  // lexer_parts ::= lexer_parts syntax_problem
				 ((List<ITmaLexerPart>)tmStack[tmHead - 1].value).add(((TmaSyntaxProblem)tmStack[tmHead].value)); 
				break;
			case 48:  // named_pattern ::= ID '=' pattern
				 lapg_gg.value = new TmaNamedPattern(((String)tmStack[tmHead - 2].value), ((TmaPattern)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 59:  // lexeme ::= identifier typeopt ':' pattern lexem_transitionopt iconopt lexem_attrsopt commandopt
				 lapg_gg.value = new TmaLexeme(((TmaIdentifier)tmStack[tmHead - 7].value), ((String)tmStack[tmHead - 6].value), ((TmaPattern)tmStack[tmHead - 4].value), ((TmaStateref)tmStack[tmHead - 3].value), ((Integer)tmStack[tmHead - 2].value), ((TmaLexemAttrs)tmStack[tmHead - 1].value), ((TmaCommand)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 60:  // lexeme ::= identifier typeopt ':'
				 lapg_gg.value = new TmaLexeme(((TmaIdentifier)tmStack[tmHead - 2].value), ((String)tmStack[tmHead - 1].value), ((TmaPattern)null), ((TmaStateref)null), ((Integer)null), ((TmaLexemAttrs)null), ((TmaCommand)null), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 61:  // lexem_transition ::= '=>' stateref
				 lapg_gg.value = ((TmaStateref)tmStack[tmHead].value); 
				break;
			case 62:  // lexem_attrs ::= '(' lexem_attribute ')'
				 lapg_gg.value = ((TmaLexemAttrs)tmStack[tmHead - 1].value); 
				break;
			case 63:  // lexem_attribute ::= Lsoft
				 lapg_gg.value = new TmaLexemAttrs(LexerRule.KIND_SOFT, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 64:  // lexem_attribute ::= Lclass
				 lapg_gg.value = new TmaLexemAttrs(LexerRule.KIND_CLASS, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 65:  // lexem_attribute ::= Lspace
				 lapg_gg.value = new TmaLexemAttrs(LexerRule.KIND_SPACE, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 66:  // lexem_attribute ::= Llayout
				 lapg_gg.value = new TmaLexemAttrs(LexerRule.KIND_LAYOUT, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 67:  // state_selector ::= '[' state_list ']'
				 lapg_gg.value = new TmaStateSelector(((List<TmaLexerState>)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 68:  // state_list ::= lexer_state
				 lapg_gg.value = new ArrayList<Integer>(4); ((List<TmaLexerState>)lapg_gg.value).add(((TmaLexerState)tmStack[tmHead].value)); 
				break;
			case 69:  // state_list ::= state_list ',' lexer_state
				 ((List<TmaLexerState>)tmStack[tmHead - 2].value).add(((TmaLexerState)tmStack[tmHead].value)); 
				break;
			case 70:  // stateref ::= ID
				 lapg_gg.value = new TmaStateref(((String)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 71:  // lexer_state ::= identifier '=>' stateref
				 lapg_gg.value = new TmaLexerState(((TmaIdentifier)tmStack[tmHead - 2].value), ((TmaStateref)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 72:  // lexer_state ::= identifier
				 lapg_gg.value = new TmaLexerState(((TmaIdentifier)tmStack[tmHead].value), ((TmaStateref)null), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 73:  // grammar_parts ::= grammar_part
				 lapg_gg.value = new ArrayList<ITmaGrammarPart>(64); ((List<ITmaGrammarPart>)lapg_gg.value).add(((ITmaGrammarPart)tmStack[tmHead].value)); 
				break;
			case 74:  // grammar_parts ::= grammar_parts grammar_part
				 ((List<ITmaGrammarPart>)tmStack[tmHead - 1].value).add(((ITmaGrammarPart)tmStack[tmHead].value)); 
				break;
			case 75:  // grammar_parts ::= grammar_parts syntax_problem
				 ((List<ITmaGrammarPart>)tmStack[tmHead - 1].value).add(((TmaSyntaxProblem)tmStack[tmHead].value)); 
				break;
			case 78:  // nonterm ::= annotations identifier nonterm_type '::=' rules ';'
				 lapg_gg.value = new TmaNonterm(((TmaIdentifier)tmStack[tmHead - 4].value), ((TmaNontermType)tmStack[tmHead - 3].value), ((List<TmaRule0>)tmStack[tmHead - 1].value), ((TmaAnnotations)tmStack[tmHead - 5].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 79:  // nonterm ::= annotations identifier '::=' rules ';'
				 lapg_gg.value = new TmaNonterm(((TmaIdentifier)tmStack[tmHead - 3].value), ((TmaNontermType)null), ((List<TmaRule0>)tmStack[tmHead - 1].value), ((TmaAnnotations)tmStack[tmHead - 4].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 80:  // nonterm ::= identifier nonterm_type '::=' rules ';'
				 lapg_gg.value = new TmaNonterm(((TmaIdentifier)tmStack[tmHead - 4].value), ((TmaNontermType)tmStack[tmHead - 3].value), ((List<TmaRule0>)tmStack[tmHead - 1].value), ((TmaAnnotations)null), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 81:  // nonterm ::= identifier '::=' rules ';'
				 lapg_gg.value = new TmaNonterm(((TmaIdentifier)tmStack[tmHead - 3].value), ((TmaNontermType)null), ((List<TmaRule0>)tmStack[tmHead - 1].value), ((TmaAnnotations)null), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 82:  // nonterm_type ::= Lreturns symref
				 lapg_gg.value = new TmaNontermTypeAST(((TmaSymref)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 85:  // nonterm_type ::= Linline Lclass identifieropt
				 lapg_gg.value = new TmaNontermTypeHint(TmaNontermTypeHint.Kind.INLINE_CLASS, ((TmaIdentifier)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 86:  // nonterm_type ::= Lclass identifieropt
				 lapg_gg.value = new TmaNontermTypeHint(TmaNontermTypeHint.Kind.CLASS, ((TmaIdentifier)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 87:  // nonterm_type ::= Linterface identifieropt
				 lapg_gg.value = new TmaNontermTypeHint(TmaNontermTypeHint.Kind.INTERFACE, ((TmaIdentifier)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 88:  // nonterm_type ::= Lvoid
				 lapg_gg.value = new TmaNontermTypeHint(TmaNontermTypeHint.Kind.VOID, null, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 89:  // nonterm_type ::= type
				 lapg_gg.value = new TmaNontermTypeRaw(((String)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 93:  // directive ::= '%' priority_kw references ';'
				 lapg_gg.value = new TmaDirectivePrio(((String)tmStack[tmHead - 2].value), ((List<TmaSymref>)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 94:  // directive ::= '%' Linput inputs ';'
				 lapg_gg.value = new TmaDirectiveInput(((List<TmaInputref>)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 95:  // inputs ::= inputref
				 lapg_gg.value = new ArrayList<TmaInputref>(); ((List<TmaInputref>)lapg_gg.value).add(((TmaInputref)tmStack[tmHead].value)); 
				break;
			case 96:  // inputs ::= inputs ',' inputref
				 ((List<TmaInputref>)tmStack[tmHead - 2].value).add(((TmaInputref)tmStack[tmHead].value)); 
				break;
			case 99:  // inputref ::= symref Lnoeoiopt
				 lapg_gg.value = new TmaInputref(((TmaSymref)tmStack[tmHead - 1].value), ((String)tmStack[tmHead].value) != null, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 100:  // references ::= symref
				 lapg_gg.value = new ArrayList<TmaSymref>(); ((List<TmaSymref>)lapg_gg.value).add(((TmaSymref)tmStack[tmHead].value)); 
				break;
			case 101:  // references ::= references symref
				 ((List<TmaSymref>)tmStack[tmHead - 1].value).add(((TmaSymref)tmStack[tmHead].value)); 
				break;
			case 102:  // references_cs ::= symref
				 lapg_gg.value = new ArrayList<TmaSymref>(); ((List<TmaSymref>)lapg_gg.value).add(((TmaSymref)tmStack[tmHead].value)); 
				break;
			case 103:  // references_cs ::= references_cs ',' symref
				 ((List<TmaSymref>)tmStack[tmHead - 2].value).add(((TmaSymref)tmStack[tmHead].value)); 
				break;
			case 105:  // rule_list ::= rule0
				 lapg_gg.value = new ArrayList<TmaRule0>(); ((List<TmaRule0>)lapg_gg.value).add(((TmaRule0)tmStack[tmHead].value)); 
				break;
			case 106:  // rule_list ::= rule_list '|' rule0
				 ((List<TmaRule0>)tmStack[tmHead - 2].value).add(((TmaRule0)tmStack[tmHead].value)); 
				break;
			case 109:  // rule0 ::= rhsPrefix rhsParts rhsSuffixopt
				 lapg_gg.value = new TmaRule0(((TmaRhsPrefix)tmStack[tmHead - 2].value), ((List<ITmaRhsPart>)tmStack[tmHead - 1].value), ((TmaRhsSuffix)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 110:  // rule0 ::= rhsPrefix rhsSuffixopt
				 lapg_gg.value = new TmaRule0(((TmaRhsPrefix)tmStack[tmHead - 1].value), ((List<ITmaRhsPart>)null), ((TmaRhsSuffix)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 111:  // rule0 ::= rhsParts rhsSuffixopt
				 lapg_gg.value = new TmaRule0(((TmaRhsPrefix)null), ((List<ITmaRhsPart>)tmStack[tmHead - 1].value), ((TmaRhsSuffix)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 112:  // rule0 ::= rhsSuffixopt
				 lapg_gg.value = new TmaRule0(((TmaRhsPrefix)null), ((List<ITmaRhsPart>)null), ((TmaRhsSuffix)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 113:  // rule0 ::= syntax_problem
				 lapg_gg.value = new TmaRule0(((TmaSyntaxProblem)tmStack[tmHead].value)); 
				break;
			case 114:  // rhsPrefix ::= '[' annotations ']'
				 lapg_gg.value = new TmaRhsPrefix(((TmaAnnotations)tmStack[tmHead - 1].value), null, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 115:  // rhsPrefix ::= '[' rhsAnnotations identifier ']'
				 lapg_gg.value = new TmaRhsPrefix(((TmaRuleAnnotations)tmStack[tmHead - 2].value), ((TmaIdentifier)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 116:  // rhsPrefix ::= '[' identifier ']'
				 lapg_gg.value = new TmaRhsPrefix(((TmaRuleAnnotations)null), ((TmaIdentifier)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 117:  // rhsSuffix ::= '%' Lprio symref
				 lapg_gg.value = new TmaRhsPrio(((TmaSymref)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 118:  // rhsSuffix ::= '%' Lshift
				 lapg_gg.value = new TmaRhsShiftClause(source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 119:  // rhsParts ::= rhsPart
				 lapg_gg.value = new ArrayList<ITmaRhsPart>(); ((List<ITmaRhsPart>)lapg_gg.value).add(((ITmaRhsPart)tmStack[tmHead].value)); 
				break;
			case 120:  // rhsParts ::= rhsParts rhsPart
				 ((List<ITmaRhsPart>)tmStack[tmHead - 1].value).add(((ITmaRhsPart)tmStack[tmHead].value)); 
				break;
			case 121:  // rhsParts ::= rhsParts syntax_problem
				 ((List<ITmaRhsPart>)tmStack[tmHead - 1].value).add(((TmaSyntaxProblem)tmStack[tmHead].value)); 
				break;
			case 126:  // rhsAnnotated ::= rhsAnnotations rhsAssignment
				 lapg_gg.value = new TmaRhsAnnotated(((TmaRuleAnnotations)tmStack[tmHead - 1].value), ((ITmaRhsPart)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 128:  // rhsAssignment ::= identifier '=' rhsOptional
				 lapg_gg.value = new TmaRhsAssignment(((TmaIdentifier)tmStack[tmHead - 2].value), ((ITmaRhsPart)tmStack[tmHead].value), false, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 129:  // rhsAssignment ::= identifier '+=' rhsOptional
				 lapg_gg.value = new TmaRhsAssignment(((TmaIdentifier)tmStack[tmHead - 2].value), ((ITmaRhsPart)tmStack[tmHead].value), true, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 131:  // rhsOptional ::= rhsCast '?'
				 lapg_gg.value = new TmaRhsQuantifier(((ITmaRhsPart)tmStack[tmHead - 1].value), TmaRhsQuantifier.KIND_OPTIONAL, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 133:  // rhsCast ::= rhsClass Las symref
				 lapg_gg.value = new TmaRhsCast(((ITmaRhsPart)tmStack[tmHead - 2].value), ((TmaSymref)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 134:  // rhsCast ::= rhsClass Las literal
				 lapg_gg.value = new TmaRhsAsLiteral(((ITmaRhsPart)tmStack[tmHead - 2].value), ((TmaExpressionLiteral)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 135:  // rhsUnordered ::= rhsPart '&' rhsPart
				 lapg_gg.value = new TmaRhsUnordered(((ITmaRhsPart)tmStack[tmHead - 2].value), ((ITmaRhsPart)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 137:  // rhsClass ::= identifier ':' rhsPrimary
				 lapg_gg.value = ((ITmaRhsPart)tmStack[tmHead].value); reporter.error(lapg_gg.offset, lapg_gg.endoffset, lapg_gg.line, "unsupported, TODO"); 
				break;
			case 138:  // rhsPrimary ::= symref
				 lapg_gg.value = new TmaRhsSymbol(((TmaSymref)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 139:  // rhsPrimary ::= '(' rules ')'
				 lapg_gg.value = new TmaRhsNested(((List<TmaRule0>)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 140:  // rhsPrimary ::= '(' rhsParts Lseparator references ')' '+'
				 lapg_gg.value = new TmaRhsList(((List<ITmaRhsPart>)tmStack[tmHead - 4].value), ((List<TmaSymref>)tmStack[tmHead - 2].value), true, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 141:  // rhsPrimary ::= '(' rhsParts Lseparator references ')' '*'
				 lapg_gg.value = new TmaRhsList(((List<ITmaRhsPart>)tmStack[tmHead - 4].value), ((List<TmaSymref>)tmStack[tmHead - 2].value), false, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 142:  // rhsPrimary ::= rhsPrimary '*'
				 lapg_gg.value = new TmaRhsQuantifier(((ITmaRhsPart)tmStack[tmHead - 1].value), TmaRhsQuantifier.KIND_ZEROORMORE, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 143:  // rhsPrimary ::= rhsPrimary '+'
				 lapg_gg.value = new TmaRhsQuantifier(((ITmaRhsPart)tmStack[tmHead - 1].value), TmaRhsQuantifier.KIND_ONEORMORE, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 144:  // rhsAnnotations ::= annotation_list
				 lapg_gg.value = new TmaRuleAnnotations(null, ((List<TmaMapEntriesItem>)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 145:  // rhsAnnotations ::= negative_la annotation_list
				 lapg_gg.value = new TmaRuleAnnotations(((TmaNegativeLa)tmStack[tmHead - 1].value), ((List<TmaMapEntriesItem>)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 146:  // rhsAnnotations ::= negative_la
				 lapg_gg.value = new TmaRuleAnnotations(((TmaNegativeLa)tmStack[tmHead].value), null, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 147:  // annotations ::= annotation_list
				 lapg_gg.value = new TmaAnnotations(((List<TmaMapEntriesItem>)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 148:  // annotation_list ::= annotation
				 lapg_gg.value = new ArrayList<TmaMapEntriesItem>(); ((List<TmaMapEntriesItem>)lapg_gg.value).add(((TmaMapEntriesItem)tmStack[tmHead].value)); 
				break;
			case 149:  // annotation_list ::= annotation_list annotation
				 ((List<TmaMapEntriesItem>)tmStack[tmHead - 1].value).add(((TmaMapEntriesItem)tmStack[tmHead].value)); 
				break;
			case 150:  // annotation ::= '@' ID '{' expression '}'
				 lapg_gg.value = new TmaMapEntriesItem(((String)tmStack[tmHead - 3].value), ((ITmaExpression)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 151:  // annotation ::= '@' ID
				 lapg_gg.value = new TmaMapEntriesItem(((String)tmStack[tmHead].value), ((ITmaExpression)null), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 152:  // annotation ::= '@' syntax_problem
				 lapg_gg.value = new TmaMapEntriesItem(((TmaSyntaxProblem)tmStack[tmHead].value)); 
				break;
			case 153:  // negative_la ::= '(?!' negative_la_clause ')'
				 lapg_gg.value = new TmaNegativeLa(((List<TmaSymref>)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 154:  // negative_la_clause ::= symref
				 lapg_gg.value = new ArrayList<TmaSymref>(); ((List<TmaSymref>)lapg_gg.value).add(((TmaSymref)tmStack[tmHead].value)); 
				break;
			case 155:  // negative_la_clause ::= negative_la_clause '|' symref
				 ((List<TmaSymref>)tmStack[tmHead - 2].value).add(((TmaSymref)tmStack[tmHead].value)); 
				break;
			case 160:  // expression ::= Lnew name '(' map_entriesopt ')'
				 lapg_gg.value = new TmaExpressionInstance(((TmaName)tmStack[tmHead - 3].value), ((List<TmaMapEntriesItem>)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 163:  // expression ::= '[' expression_listopt ']'
				 lapg_gg.value = new TmaExpressionArray(((List<ITmaExpression>)tmStack[tmHead - 1].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 165:  // literal ::= scon
				 lapg_gg.value = new TmaExpressionLiteral(((String)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 166:  // literal ::= icon
				 lapg_gg.value = new TmaExpressionLiteral(((Integer)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 167:  // literal ::= Ltrue
				 lapg_gg.value = new TmaExpressionLiteral(Boolean.TRUE, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 168:  // literal ::= Lfalse
				 lapg_gg.value = new TmaExpressionLiteral(Boolean.FALSE, source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 169:  // expression_list ::= expression
				 lapg_gg.value = new ArrayList(); ((List<ITmaExpression>)lapg_gg.value).add(((ITmaExpression)tmStack[tmHead].value)); 
				break;
			case 170:  // expression_list ::= expression_list ',' expression
				 ((List<ITmaExpression>)tmStack[tmHead - 2].value).add(((ITmaExpression)tmStack[tmHead].value)); 
				break;
			case 171:  // map_entries ::= ID map_separator expression
				 lapg_gg.value = new ArrayList<TmaMapEntriesItem>(); ((List<TmaMapEntriesItem>)lapg_gg.value).add(new TmaMapEntriesItem(((String)tmStack[tmHead - 2].value), ((ITmaExpression)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset)); 
				break;
			case 172:  // map_entries ::= map_entries ',' ID map_separator expression
				 ((List<TmaMapEntriesItem>)tmStack[tmHead - 4].value).add(new TmaMapEntriesItem(((String)tmStack[tmHead - 2].value), ((ITmaExpression)tmStack[tmHead].value), source, tmStack[tmHead - 2].offset, lapg_gg.endoffset)); 
				break;
			case 176:  // name ::= qualified_id
				 lapg_gg.value = new TmaName(((String)tmStack[tmHead].value), source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
			case 178:  // qualified_id ::= qualified_id '.' ID
				 lapg_gg.value = ((String)tmStack[tmHead - 2].value) + "." + ((String)tmStack[tmHead].value); 
				break;
			case 179:  // command ::= code
				 lapg_gg.value = new TmaCommand(source, tmStack[tmHead].offset+1, tmStack[tmHead].endoffset-1); 
				break;
			case 180:  // syntax_problem ::= error
				 lapg_gg.value = new TmaSyntaxProblem(source, lapg_gg.offset, lapg_gg.endoffset); 
				break;
		}
	}

	/**
	 * disposes symbol dropped by error recovery mechanism
	 */
	protected void dispose(LapgSymbol value) {
	}

	/**
	 * cleans node removed from the stack
	 */
	protected void cleanup(LapgSymbol value) {
	}

	public TmaInput parseInput(TMLexer lexer) throws IOException, ParseException {
		return (TmaInput) parse(lexer, 0, 284);
	}

	public ITmaExpression parseExpression(TMLexer lexer) throws IOException, ParseException {
		return (ITmaExpression) parse(lexer, 1, 285);
	}
}
